
'-- Demo of finding a selected listbox item. The file jsShell Demo.exe is a sample program for use with this script.
'
'-- NOTES:  This script uses jsShell, which uses Active Accessibility, to find coordinates and caption of the selected
' item in a listbox. Note, however, that this is not an easy task. If you have a window open that has a listbox in
' the window, the script can find the selectd item given the parent window title. But you'll find that the operation
' fails in many windows because the listbox is not on the top-level window. With AA, things can be unpredictable
' because software GUIs can be constructed in many ways. If you have a listbox on a frame or tab, that will be
' a subwindow. The script can still find the listbox, but you must first get to the subwindow window handle. For
' example, the Outlook Express Options applet has 6-7 tabs and some buttons. This script only sees the buttons, etc.
' in the parent window. If the General tab is displayed there will be a subwindow with the title of "General".
' You would need to use GetChildWindows on the parent window to find the tab window.

' If you use AccExplorer you'll see that window structure can be extremely complex. On the other hand, it can
' also be extremely simple, which is also a problem. For example, the interface of Firefox is mainly owner-drawn.
' There are few "windows". Since the address bar is not a window it is not visible to AA. AA can be very useful
' if you know the window you will need to work with, but in many cases you will need to customize the script to
' work with a specific window.


Dim jsS, jsSWins, i2, i3, iCount, A1, sTitle, H1, H2, L1, T1, W1, Ht1, sList
Dim jsSAAWin, jsSAAObj, jsSAAWinLB, jsSAAObjLI, sCaption, TitleToFind

MsgBox "This script demonstrates finding the selected item in a listbox, in a given window. It does not check for a specific listbox. It just picks the first top-level one it finds in the window. (See notes in script.) To use the demo program for this script, open jsShell Demo.exe now.", 64

TitleToFind = InputBox( "Enter a unique portion, or all, of the window title on the window you want to access. If you are using the demo program, enter 'AA Sample'", "Enter window title.")
  If Len(TitleToFind) = 0 Then WScript.quit
  
Set jsS = CreateObject("jsShell.Ops")
 Set jsSWins = jsS.GetOpenWindows

i2 = jsSWins.GetOpenWindowTitles(A1)
  If i2 > 0 Then
    For i3 = 0 to i2 - 1
        sTitle = A1(i3)
        If InStr(1, sTitle, TitleToFind, 1) > 0 Then Exit For
    Next
  End If
  
 If InStr(1, sTitle, TitleToFind, 1) = 0 Then MsgBox "Window not found,", 64 :  DropIt '--window not found.
  
H1 = jsSWins.GetWindowHandleFromTitle(sTitle) '-- get VB form1 window handle.

Set jsSAAWin = jsS.GetAAWindowByHandle(H1) '-- get data from iaccessible pointer.

iCount = jsSAAWin.ItemCount  'window object count


For i2 = 0 to iCount - 1

  '-- uncomment this to see a list of objects found in the top-level window.
 '  sList = sList & jsS.GetAAObjectNameFromObjectID(jsSAAWin.Item(i2).ObjectID) & vbCrLf  '-- <<<extra demo commented out>>>
   
 '-- when a listbox is found in window, get it as an AAObject.
  If jsSAAWin.Item(i2).ObjectID = 33 Then 
      Set jsSAAObj = jsSAAWin.Item(i2)     
       Exit For
   End If
 Next   

 '-- uncomment this, along with line above, to see list of window objects.
 ' MsgBox sList                                                                                                                      '-- <<<extra demo commented out>>>

If IsObject(jsSAAObj) = False Then MsgBox "No listbox found in window.", 64 : DropIt

H2 = jsSAAObj.Handle '-- get listbox handle.
  '-- get listbox window as AAWindow.
Set jsSAAWinLB = jsS.GetAAWindowByHandle(H2) 

 iCount = jsSAAWinLB.ItemCount 'object count in listbox.

For i2 = 0 to iCount - 1
    With jsSAAWinLB.Item(i2)
      If .ObjectID = 34 Then  'listitem
         If  .Selected = True Then  
            L1 = .PixLeft 
            T1 = .PixTop
            W1 = .PixWide 
            Ht1 = .PixHigh
            sCaption = .Caption
           Exit For
        End If     
      End If
    End With  
Next     
  '-- coordinates of selected item.
  If Len(sCaption) > 0 Then
      MsgBox "Selected item screen coordinates (for use with automated mouse clicks, etc.): " & L1 & "-" & T1 & "-" & W1 & "-" & Ht1 & vbCrLf & "Listitem text: " & sCaption
  Else
     MsgBox "No selectected item found in listbox."
 End If        

DropIt

Sub DropIt()
  Set jsSAAWinLB = Nothing
  Set jsSAAObj = Nothing
  Set jsSAAWin = Nothing
  Set jsSWins = Nothing
  Set jsS = Nothing
  WScript.Quit
End Sub  